/*  Project listview
    Chinchilla Soft Ltd
    Copyright  1995. All Rights Reserved.

    SUBSYSTEM:    listview.exe Application
    FILE:         lstvwwnd.cpp
    AUTHOR:       Johannes Plachy


    OVERVIEW
    ========
    Source file for implementation of listviewWindow (TWindow).
*/


#include <owl\owlpch.h>
#pragma hdrstop

#include "lstvwapp.h"
#include "lstvwwnd.h"
#include "lstvwapp.rh"

#include <stdio.h>

#define NUM_COLUMNS 5
#define NUM_ITEMS 120
#define MAX_ITEMLEN 150
#define MAX_ICONS 4


DEFINE_RESPONSE_TABLE1(TListViewWindow, TDecoratedFrame)

    EV_MESSAGE( WM_NOTIFY ,ContainerNotification),

    EV_COMMAND_AND_ID( IDM_LARGEICON, ChangeView),
    EV_COMMAND_AND_ID( IDM_SMALLICON, ChangeView),
    EV_COMMAND_AND_ID( IDM_LISTVIEW, ChangeView),
    EV_COMMAND_AND_ID( IDM_REPORTVIEW, ChangeView),

END_RESPONSE_TABLE;


//---------------------------------------------------------------------------

TListViewWindow::TListViewWindow (TWindow* Parent, const char far* title,
                  TWindow* clientWnd ,
                  bool shrinkToClient,
                  TModule* module)
    : TDecoratedFrame(Parent, title,
#ifdef TREEVIEW_EXMAPLE
      (clientWnd ? clientWnd : pView = new TTreeView( 0,1,0,0,0,0,0)),
#else
      (clientWnd ? clientWnd : pView = new TContainer( 0,1,0,0,0,0,0,LVS_REPORT/*LVS_ICON*/)),
#endif
      shrinkToClient,
      module)
{
   int cx = GetSystemMetrics( SM_CXICON);
   int cy = GetSystemMetrics( SM_CYICON);

   pIconImageList = new TImageList( cx, cy, ILC_COLOR16, 10, 10, FALSE);
   pBmpImageList = new TImageList( 20, 20, ILC_COLOR16, 10, 10, FALSE);
}

//---------------------------------------------------------------------------

TListViewWindow::~TListViewWindow ()
{
    delete pIconImageList;
    delete pBmpImageList;
}

//---------------------------------------------------------------------------

void TListViewWindow::SetupWindow()
{
   TDecoratedFrame::SetupWindow();

   for (int i = 1; i <= MAX_ICONS; i++)
   {
      pBmpImageList->Add( GetApplication()->LoadBitmap ( MAKEINTRESOURCE(i)));
#ifndef TREEVIEW_EXMAPLE
      pIconImageList->AddIcon( GetApplication()->LoadIcon ( MAKEINTRESOURCE(i)));
#endif
   }

#ifdef TREEVIEW_EXMAPLE
   pView->SetImageList( *pBmpImageList, TVSIL_NORMAL);

   HTREEITEM DataBaseInsertAfter = TVI_ROOT;
   HTREEITEM TemplateInsertAfter = TVI_ROOT;
   HTREEITEM ProcessesInsertAfter = TVI_ROOT;
   HTREEITEM ActivitiesInsertAfter = TVI_ROOT;

   for ( int databases = 0; databases < 5; databases++)
   {
      char szText[64];
      wsprintf( szText, "Database # %d",databases);

      DataBaseInsertAfter = AddTreeItem( DataBaseInsertAfter, (HTREEITEM)TVI_ROOT, szText, 0 );

      TemplateInsertAfter = DataBaseInsertAfter;

      for ( int templates = 0; templates < 5; templates++)
      {
         char szText[64];
         wsprintf( szText, "Template # %d in database %d",templates, databases);

         TemplateInsertAfter = AddTreeItem( TemplateInsertAfter, DataBaseInsertAfter, szText, 1 );

         ProcessesInsertAfter = TemplateInsertAfter;
         for ( int processes = 0; processes < 5; processes++)
         {
            char szText[64];
            wsprintf( szText, "Process # %d of template %d",processes,templates);

            ProcessesInsertAfter = AddTreeItem( ProcessesInsertAfter, TemplateInsertAfter, szText, 2 );
            ActivitiesInsertAfter = ProcessesInsertAfter;

            for ( int activities = 0; activities < 5; activities++)
            {
               char szText[64];
               wsprintf( szText, "Activity # %d of process %d",activities,processes);

               ActivitiesInsertAfter = AddTreeItem( ActivitiesInsertAfter, ProcessesInsertAfter, szText, 3 );
            }
         }
      }
   }
#else
   pView->SetImageList( *pIconImageList);
   pView->SetImageList( *pBmpImageList, LVSIL_SMALL);

	LV_COLUMN lvC;      // List View Column structure
	char szText[64];    // place to store some text
	LV_ITEM lvI;        // List view item structure


 	lvC.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvC.fmt = LVCFMT_LEFT;  // left align the column
	lvC.cx = 75;            // width of the column, in pixels
	lvC.pszText = szText;

	// Add the columns.
	for (int index = 1; index <= NUM_COLUMNS; index++)
	{
		lvC.iSubItem = index;
		GetApplication()->LoadString(
                                    index,
                                    szText,
                                    sizeof(szText));
		if (pView->InsertColumn( index, &lvC) == -1)
			return ;
	}

	// Finally, let's add the actual items to the control
	// Fill in the LV_ITEM structure for each of the items to add
	// to the list.
	// The mask specifies the the .pszText, .iImage, .lParam and .state
	// members of the LV_ITEM structure are valid.
	lvI.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE;
	lvI.state = 0;      //
	lvI.stateMask = 0;  //

	for (index = 1; index <= NUM_ITEMS; index++)
	{
		lvI.iItem = index;
		lvI.iSubItem = 0;
		// The parent window is responsible for storing the text. The List view
		// window will send a LVN_GETDISPINFO when it needs the text to display/
		lvI.pszText = LPSTR_TEXTCALLBACK;
		lvI.cchTextMax = MAX_ITEMLEN;
		lvI.iImage = index % MAX_ICONS;
		lvI.lParam = index;

		if (pView->InsertItem( &lvI) == -1)
			return ;

		for (int iSubItem = 1; iSubItem < NUM_COLUMNS; iSubItem++)
		{
			pView->SetItemText(index,
				                     iSubItem,
				                     (LPCSTR)LPSTR_TEXTCALLBACK);
		}
	}
#endif
}

//---------------------------------------------------------------------------

HTREEITEM TListViewWindow::AddTreeItem( HTREEITEM ItemToInsertAfter, HTREEITEM ParentItem, LPSTR text, int iImage )
{
#ifdef TREEVIEW_EXMAPLE
	TV_ITEM tvI;
	TV_INSERTSTRUCT tvIns;

   tvI.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
   tvI.pszText = text;
   tvI.cchTextMax = lstrlen(tvI.pszText);
   tvI.iImage = iImage;
   tvI.iSelectedImage = iImage;
   tvI.lParam = 1;             // dont!! save the index of the item
   tvIns.item = tvI;
   tvIns.hInsertAfter = ItemToInsertAfter;
   tvIns.hParent = ParentItem;

   return pView->InsertItem( &tvIns);
#else
   return NULL;
#endif
}

//---------------------------------------------------------------------------

LRESULT TListViewWindow::ContainerNotification( WPARAM wParam, LPARAM lParam)
{
   LV_DISPINFO *pLvdi = (LV_DISPINFO *)lParam;
#ifndef TREEVIEW_EXMAPLE
   static char szText[10];
#endif

   switch(pLvdi->hdr.code)
   {
#ifndef TREEVIEW_EXMAPLE
      case LVN_GETDISPINFO:
         switch (pLvdi->item.iSubItem) // means columns !!
         {
            default:
               wsprintf( szText, " This a text for Column %d for Item %d", pLvdi->item.iSubItem+1, pLvdi->item.lParam);
               pLvdi->item.pszText = szText;
  				break;
          }
          break;
      case LVN_COLUMNCLICK:
         MessageBeep( MB_OK);
         break;

      // selection notifications and others
      case NM_CLICK :
      case NM_RCLICK :
      case NM_DBLCLK :
      case NM_RDBLCLK :
      case NM_RETURN :
      case NM_SETFOCUS :
      case NM_KILLFOCUS :
      case NM_STARTWAIT :
      case NM_ENDWAIT :
      case NM_OUTOFMEMORY :

      // list control specific

      case LVN_BEGINDRAG :
      case LVN_BEGINRDRAG :
      case LVN_ENDDRAG :
      case LVN_ENDRDRAG :
      case LVN_BEGINLABELEDIT :
      case LVN_ENDLABELEDIT :
      case LVN_DELETEALLITEMS :
      case LVN_INSERTITEM :
      case LVN_DELETEITEM :
      case LVN_ITEMCHANGED :
      case LVN_ITEMCHANGING :
      case LVN_KEYDOWN :
      case LVN_SETDISPINFO :
#else
#endif

      default:
         break;
   }
   return 0L;
}

//---------------------------------------------------------------------------

void TListViewWindow::ChangeView( WPARAM Command)
{
#ifndef TREEVIEW_EXMAPLE
   switch ( LOWORD( Command ))
   {
      case IDM_LARGEICON :
         pView->ChangeView( LVS_ICON);
         break;
      case IDM_SMALLICON :
         pView->ChangeView( LVS_SMALLICON);
         break;
      case IDM_LISTVIEW  :
         pView->ChangeView( LVS_LIST);
         break;
      case IDM_REPORTVIEW:
         pView->ChangeView( LVS_REPORT);
         break;
      default :
         break;
   }
#else
   MessageBeep(0);
#endif
}

//---------------------------------------------------------------------------
